/**************************************************************************
ColourizeSmooth uses a general colourizing algorithm to smooth a given
clip.  Algorithm is based at http://www.cs.huji.ac.il/~yweiss/Colorization.
Copyright (C) 2005 Aditya Bhargava

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**************************************************************************/

#define _USE_MATH_DEFINES

#include <climits>
#include <cmath>

#include <windows.h>

#include "internal.h"

class ColourizeSmooth : public GenericVideoFilter
{
private:
	bool sup, gaussian;
	int radius, iterations, threshold;

	int width, height;
	unsigned char *pBuffer1, *pBuffer2;

	bool areNeighbours(int x1, int y1, int x2, int y2);
	void copyBuffer(IScriptEnvironment* env);
	int doIteration();
	double weighter(unsigned char* p1, int x1, unsigned char* p2, int x2, double mean, double variance);
	void YUVToYUYV(PVideoFrame& dst);
	void YUYVToYUV(PVideoFrame& src);

public:
	ColourizeSmooth(PClip _child, bool _sup, int _radius, bool _gaussian, int _iterations, int _threshold,
		IScriptEnvironment* env) : GenericVideoFilter(_child), sup(_sup), radius(_radius), gaussian(_gaussian),
		iterations(_iterations), threshold(_threshold)
	{
		if (!vi.IsYUY2() && !vi.IsYV12())
			env->ThrowError("ColourizeSmooth: YUY2 or YV12 data only.");
		if (radius < 1)
			env->ThrowError("ColourizeSmooth: \"radius\" parameter must be greater than or equal to one.");
		if (iterations < 0)
			env->ThrowError("ColourizeSmooth: \"iterations\" parameter must be greater than or equal to zero.");
		if ((threshold < 0) || (threshold > 255))
			env->ThrowError("ColourizeSmooth: \"threshold\" parameter must be between zero and 255 inclusive.");

		width = vi.width;
		height = vi.height;

		pBuffer1 = new unsigned char[height * width * 3];
		pBuffer2 = new unsigned char[height * width * 3];
	}

	~ColourizeSmooth()
	{
		if (pBuffer1 != NULL)
			delete [] pBuffer1;
		if (pBuffer2 != NULL)
			delete [] pBuffer2;
	}

	PVideoFrame __stdcall GetFrame(int n, IScriptEnvironment* env);
};